From c91b8393a4cff5ea6eacff4f5bb281cddf54c94c Mon Sep 17 00:00:00 2001
From: David Powell <djpowell@djpowell.net>
Date: Sat, 5 Jun 2010 16:18:19 +0100
Subject: [PATCH 3/6] use clojure.java.io to copy streams, avoiding byte-at-a-time copying

---
 src/clj/clojure/java/shell.clj |   21 ++++++++++++++++-----
 1 files changed, 16 insertions(+), 5 deletions(-)

diff --git a/src/clj/clojure/java/shell.clj b/src/clj/clojure/java/shell.clj
index 237e6ba..58d2432 100644
--- a/src/clj/clojure/java/shell.clj
+++ b/src/clj/clojure/java/shell.clj
@@ -11,8 +11,8 @@
     :doc "Conveniently launch a sub-process providing its stdin and
 collecting its stdout"}
   clojure.java.shell
-  (:use [clojure.java.io :only (as-file)])
-  (:import (java.io InputStreamReader OutputStreamWriter)))
+  (:use [clojure.java.io :only (as-file copy)])
+  (:import (java.io OutputStreamWriter ByteArrayOutputStream StringWriter)))
 
 (def *sh-dir* nil)
 (def *sh-env* nil)
@@ -60,6 +60,18 @@ collecting its stdout"}
    (map? arg) (into-array String (map (fn [[k v]] (str (name k) "=" v)) arg))
    true arg))
 
+(defn- stream-to-bytes
+  [in]
+  (with-open [bout (ByteArrayOutputStream.)]
+    (copy in bout)
+    (.toByteArray bout)))
+
+(defn- stream-to-string
+  [in enc]
+  (with-open [bout (StringWriter.)]
+    (copy in bout :encoding enc)
+    (.toString bout)))
+
 (defn sh
   "Passes the given strings to Runtime.exec() to launch a sub-process.
 
@@ -99,9 +111,8 @@ collecting its stdout"}
                 stderr (.getErrorStream proc)]
       (let [[out err]
             (if (= (:out opts) :bytes)
-              (pmap #(into-array Byte/TYPE (map byte (stream-seq %))) [stdout stderr])
-              (pmap #(apply str (map char (stream-seq 
-                                      (InputStreamReader. % (:out opts))))) [stdout stderr]))
+              (pmap #(stream-to-bytes %) [stdout stderr])
+              (pmap #(stream-to-string % (:out opts)) [stdout stderr]))
             exit-code (.waitFor proc)]
         {:exit exit-code :out out :err err}))))
 
-- 
1.6.5.1.1367.gcd48

